#!/bin/bash
# photodrop - This daemon script creates on the fly a dynamic html
# photoalbum website from photo's dropped in the "dropfolder" 
# directory. This dropfolder directory is automaticly created in the
# main folder that was given as an argument when starting this
# script (when it doesn't exist yet). In the main folder can also be
# the automaticly created index.html file.
#
# A lot of image and file formats are supported. The script makes use
# of ffmpeg and imagemagick and notify-tools packages, these must be
# installed before using the script.
# 
# Usage: photodrop path/to/photodrop/folder
# Then pictures can be dropped in the dropfolder. The script wil 
# automaticly create and update a website that can be accessed with 
# the index.html file in the main folder.

# declare variables
desktopfolder="$(xdg-user-dir DESKTOP)"
u1folder="$(xdg-user-dir)/Ubuntu One"                  # Ubuntu One folder 
mainfolder="${u1folder}/Photodrop"                  # place where index.html and subdirectories
dropfolder="${mainfolder}/dropfolder"   # "listening" folder, where files are dropped
webfolder="${mainfolder}/htmlfiles"     # folder where html pages are created and updated
thumbfolder="${mainfolder}/images"      # folder where resized images are stored
fileindex=""                            # place in array of current file
filetotal=""                            # total number of files
notifile=""                             # string with feedback inotifywait
dropfilefolder=""                       # folder in of dropped filee
dropfilename=""                         # name of dropped file
imagename=""                            # name of (identified) image
filebasename=""                         # name without extension and dot replaced by dash
filearray=("")
commandstatus=""                        # temp var to store exit status command
searchstring=""                         # temp var to store search string

# ========= Function definitions ===================

# errout
function errout ()
{
    printf "%b\n" "$@"
} >&2

function usage ()
{
echo
echo 'After starting the script pictures may be dropped in the dropfolder that 
was automaticly created by the script inside the main folder given as 
an argument to the script. Inside the main folder the index.html file can 
be found that gives access to the dynamicly created photoalbum website. 
Video files are also suported and are shown as a animated gif thumb. 
Clicking the refresh button will load any new pictures dropped in the 
dropfolder.' 
echo
}


# emit ()
function emit ()
{
cat <<EOF
<HTML>
<HEAD><TITLE>${dropfilename}</TITLE></HEAD>
<BODY>
<TABLE WIDTH="700">
  <TR>
  <TD WIDTH="56%" ALIGN="left"> ${dropfilename} </TD>
  <TD WIDTH="8%"> <A HREF="${firsthtmlfile}"> First </A> </TD>
  <TD WIDTH="8%"> <A HREF="${prevhtmlfile}"> Prev </A> </TD>
  <TD WIDTH="12%"> <A HREF="index.html"> Refresh </A> </TD>
  <TD WIDTH="8%"> <A HREF="${nexthtmlfile}"> Next </A> </TD>
  <TD WIDTH="8%"> <A HREF="${lasthtmlfile}"> Last </A> </TD>
  </TR>
</TABLE>
  <A HREF="${image_url}">
  <IMG SRC="${thumb_url}"
  BORDER="1" VSPACE="4" HSPACE="4"/>
  </A>
</BODY>
</HTML>
EOF
}

# indexfiles (filename)  Put files in array, and find index filename
function indexfiles ()
{
local temp
local count
count=0
filearray=( )
ifs="${IFS}"
IFS=$'\n'
for temp in $(ls "-1" "${thumbfolder}")
do
    (( count+=1 ))
    filearray[$count]="${temp%.*}"
    if [[ "${temp}" == "${imagename}" ]]
    then
        fileindex="$count"
        echo "count: ${count} ${temp} *"
    else 
        echo "count: ${count} ${temp}"
    fi
done

filetotal=$count
IFS="${ifs}"
}


# Create empty page for when no images are to be shown
function emptypage ()
{
cat <<EOF
<html>
<head>
<title>Photodrop</title>
<style type="text/css">
  div.thumb {
    float:left;
    margin:5px;
    width:160px;
    height:120px;
    background-color:#aaaaaa;
  }
</style>
</head>
<body onLoad=window.refresh>
<!-- start -->
<!-- end -->
</body>
</html>
EOF
}>"${webfolder}/index.html"

function indexpage ()
{
cat <<EOF
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
<title>Photodrop</title>
<meta http-equiv="REFRESH" content="0;url=./htmlfiles">
</HEAD>
<BODY>
</BODY>
</HTML>
EOF
}>"${mainfolder}/index.html"

#============= Initialize ================
# get options 
while getopts 'h(help)' option
do
    case $option in
    h)  echo "Usage: $(basename $0)";usage;exit 0 ;;
    ?)  errout "Usage: $(basename $0) [-h]";exit 1 ;;
    esac
done
shift $(($OPTIND -1))

# evaluate argument
if [ ! -w "$u1folder" ]
then
    errout "Directory ${u1folder} is not writable."
    exit 1
fi
if [ ! -d "$mainfolder" ]
then
  mkdir "$mainfolder"
fi
if [ ! -w "$mainfolder" ]
then
    errout "Directory ${mainfolder} is not writable."
    exit 1
fi

# make subdirectories if they do not exist
if [ ! -d "${thumbfolder}" ]
then
    mkdir "${thumbfolder}"
fi
if [ ! -d "${dropfolder}" ]
then
    mkdir "${dropfolder}"
fi
if [ ! -d "${webfolder}" ]
then
    mkdir "${webfolder}"
fi

# create desktop link to dropfolder
if [ ! -d "${desktopfolder}/U1 Photodrop" ]; then
  ln -s "${dropfolder}" "${desktopfolder}/U1 Photodrop"
  gvfs-set-attribute "${desktopfolder}/U1 Photodrop" metadata::custom-icon "file:///$(cd "$(dirname "$0")" && pwd)/icon.svg"
  echo "Created a dropfolder on desktop."
fi

# create desktop file to show album site
if [ ! -f "${desktopfolder}/albumlink.desktop" ]; then
cat <<EOF > "${desktopfolder}/albumlink.desktop"
[Desktop Entry]
Version=1.0
Type=Application
Terminal=false
Exec=$(cd "$(dirname "$0")" && pwd)/showlink.sh
Icon=$(cd "$(dirname "$0")" && pwd)/web.svg
Name=U1 Album Link
EOF
chmod +x "${desktopfolder}/albumlink.desktop"
echo "Created link to web album on desktop."
fi
  
# create index.html
if [ ! -f "${webfolder}/index.html" ]
then
    emptypage
fi
indexpage

#============= Main loop =======================

# Watch for new images or deleted images
echo
echo 'Watching dropfolder for events..'
inotifywait -mq -e create,delete,move --format "%w%f:%e" "${dropfolder}" |\
while read notifile; do
    echo
    echo "${notifile}"
    temp="${notifile%:*}"
    dropfilefolder="${temp%/*}"
    dropfilename="${temp##*/}"
    # replace "." with "-" in name 
    filebasename="${dropfilename%.*}-${dropfilename##*.}"
    # skip temporary files, hidden files, and files that are not in dropfolder subdirectories
    if [[ "${dropfilename}" == ~*.tmp || "${dropfilefolder}" != "${dropfolder}" || "${dropfilename}" =~ ^\..*$ ]]
    then
        echo 'Nothing done..'
        continue
    fi
    # check if event is newfile  or removed file and act accordingly
    if [[ "${notifile}" == *":CREATE" || "${notifile}" == *":MOVED_TO" ]]
    then
        # NEWFILE: MAKE THUMB
        #------------------------------------------------------------------------
        # check if file copy is completed
        printf "%s" "Waiting for file copy to finish."
        temp="y"
        temp2="x"
        while [[ "$temp2" != "$temp" ]]
        do
            { temp="$(stat -c %s "${dropfilefolder}/${dropfilename}")" ;} > /dev/null 2>&1
            # check if file is removed while copying
            if (( $? ))
            then
                break 
            fi
            printf "%s" "."
            sleep 0.5
            { temp2="$(stat -c %s "${dropfilefolder}/${dropfilename}")" ;} > /dev/null 2>&1
            if (( $? ))
            then
                break
            fi
        done
        # skip rest if there was a problem with the file being copied..
        if [[ $temp != $temp2 ]]
        then
            echo 'nothing done (stat error during copy)..'
            continue
        fi
        printf "\n"
        # check if the dropped file is not an image
        if [[ ! "$(file "${dropfilefolder}/${dropfilename}")" =~ ^.*(PC bitmap|GIF|JPEG|Photoshop|PBM|PGM|PNG|PPM|Sun raster image|Targa image|TIFF|XCF).*$ ]]
        then
            # try to convert file with ffmpeg to animated gif
             ffmpeg -y -i "${dropfilefolder}/${dropfilename}" -pix_fmt rgb24 -r 2 -loop_output 0 -ss 0\
                    -t 5 -s 160x120 -f gif "${thumbfolder}/${filebasename}.gif" > /dev/null 2>&1
             commandstatus=$?
             # check if animated gif creation was a success
             if (( ! $commandstatus ))
             then
                 # set name of image to gif
                 imagename="${filebasename}.gif"
             else
                 imagename="${filebasename}.jpg"
                 # cleanup
                 rm -f "${thumbfolder}/${filebasename}.gif"
                 # try to convert to jpg with ffmpeg
                 ffmpeg -y -i "${dropfilefolder}/${dropfilename}" -f mjpeg -ss 0 -vframes 1\
                        -s 160x120 -an "${thumbfolder}/${imagename}" > /dev/null 2>&1
                 commandstatus=$?
             fi
        else
            imagename="${filebasename}.jpg"
            # if picture is an image, act as if ffmpeg conversion failed so that imagemagick conversion will be tried
            commandstatus=1
        fi
        # check if conversion with ffmpeg was not succesfull
        if (( ! $commandstatus ))
        then
            echo 'Converted by ffmpeg..'
        else
            rm -f "${thumbfolder}/${imagename}"
            # check if file can be converted with imagemagick
            identify "${dropfilefolder}/${dropfilename}" > /dev/null
            if (( $? ))
            then
                echo 'Nothing done..'
                continue
            else
                # convert file with imagemagick to jpg image
                convert "${dropfolder}/${dropfilename}" -resize 160x120\> "${thumbfolder}/${imagename}"
                echo 'Converted by imagemagick..'
            fi
        fi
        sleep 2
        # NEWFILE: MODIFY HTML PAGE
        #------------------------------------------------------------------------
        # get image url
        image_url=$(u1sdtool "--publish-file=${dropfolder}/${dropfilename}")
        image_url=${image_url#File\ is\ published\ at\ }
        echo "Image url: ${image_url}"
        thumb_url=$(u1sdtool "--publish-file=${thumbfolder}/${imagename}")
        thumb_url=${thumb_url#File\ is\ published\ at\ }
        echo "Thumb url: ${thumb_url}"
        indexfiles
        # modify html file
        #--------------------------
        # TO DO
        # Get insert line
        if [[ $fileindex == 1 ]]; then
          searchstring="<!-- start -->"
        else
          searchstring="${filearray[$(($fileindex - 1))]}"
        fi
        echo "Searchstring: ${searchstring}"
        line=$(($(grep -n "$searchstring" "${webfolder}/index.html"|cut -f1 -d:)+1))
        echo "Linenumber: ${line}"
        sed -i "${line}i\\<!-- image: ${filearray[$(($fileindex))]} --><div class=\"thumb\"><a href=\"${image_url}\"><img src=\"${thumb_url}\"/></a></div>" "${webfolder}/index.html" 
        #---------------------------
    # check if event is gone file and act accordingly
    elif [[ "${notifile}" == *":DELETE" || "${notifile}" == *":MOVED_FROM" ]]
    then
        # REMOVED FILE: CLEANUP
        # ----------------------------------------------------------------------------
        # check if a picture or video page from index is removed
        if [ -f "${thumbfolder}/${filebasename}.jpg" ]
        then
            imagename="${filebasename}.jpg"
        elif [ -f "${thumbfolder}/${filebasename}.gif" ]
        then
            imagename="${filebasename}.gif"
        else 
            echo 'Nothing done..'
            continue
        fi
        echo "Image ${imagename} was removed, now cleaning up the rest.."
        indexfiles
        # modify html file
        #--------------------------
        sed -i "/${filearray[$(($fileindex))]}/d" "${webfolder}/index.html"
        #---------------------------
        # cleanup thumbs 
        rm -f "${thumbfolder}/${imagename}"
    else
        # if no file was added or removed do nothing (directory events, temporarty files)
        echo 'Nothing done..'
    fi
    echo
    echo 'Watching dropfolder for events..'
#} &
done
exit 0
